#' @title Perfom hydraulic analysis for a model
#' 
#' @description
#' `analyse_hydraulic()` perfoms hydraulic analysis for a model.
#'
#' @param model either a character, name given to the model or an object of
#' 		class model
#' @param limnigraph numeric, matrix of floodwater depth heights by instant t
#' 		for each exterior opening exposed to flooding
#' @param flood_duration numeric, duration of the flood event in seconds,
#' 		optional; if not provided, calculated using limnigraph
#' @param opening_scenario character, one of the following options: "open",
#' 		"closed", "combined". See details.
#' @param dt_max numeric, optional; maxmimum timestep for simulation
#' @param Cd numeric, optional; discharge coefficient for each opening,
#' 		default to getOption("floodam_building_hydraulic_discharge_coefficient")
#' @param sim_id character, optional; id of simulation if function called from a
#' 		loop
#' @param stage character, what are the stages that should be done, default to nothing
#' @param what character, outputs to be saved
#' @param verbose boolean, will floodam tells what it is doing, default to
#' 		getOption("floodam_building_verbose")
#'
#' @details
#'
#' `opening_scenario` controls the state of openings:
#' 
#' - "open": scenario where all openings are open
#' - "closed": scenario where all openings are closed
#' - "combined": scenario where all exterior openings are closed and all interior
#' openings are open
#'
#' If `flood_duration` is missing, it is computed from `limnigraph` (max time +
#'  half an hour).
#' 
#' @return
#' 
#' list of matrix:
#' 
#' - `h`: water depths in each rooms for each time step
#' - `eQ`: water exchanges through openings for each time step
#' - `v`: velocity of exchanges through openings for each time step
#' - `eS`: wet surfaceof exchanges through openings for each time step
#' 
#' @examples
#' 
#' # declaring input and output paths
#' model_path = list(
#' 	data = system.file("extdata", package = "floodam.building"),
#' 	output = tempdir()
#' )
#' 
#' # analyzing model 'adu_t' of type 'adu' using the sequential steps *load* 
#' # (load model data), *extract* (extract model data) and *hydraulic* 
#' model = analyse_model(
#' 	model = "adu_t", 
#' 	type = "adu", 
#' 	stage = c("load", "extract", "hydraulic"), 
#' 	path = model_path
#' )
#' 
#' #' # generate limnigraph
#' limnigraph = generate_limnigraph(
#' 	time = c(0, 5400, 10800),
#'  	depth = cbind(external_1 = c(0, 3, 0), external_2 = c(0, 0, 0)),
#' 		external = list(
#' 			external_1 = c("door1", "window1", "window2"),
#' 			external_2 = c("window3")
#' 		)
#' )
#' 
#' hydraulic = analyse_hydraulic(
#' 	model = model,
#' 	limnigraph = limnigraph,
#' 	sim_id = "test",
#' 	stage = "hydraulic"
#' )
#'
#' @export

analyse_hydraulic = function(
		model,
		limnigraph,
		flood_duration,
		opening_scenario = c("combined", "close", "open"),
		dt_max = 0.5,
		Cd = 0.42,
		sim_id = NULL,
		stage = "",
		what = c("h", "eQ", "v"),
		verbose = getOption("floodam_building_verbose")
	)
{
	start = Sys.time()
	if (identical(stage, "all")) stage = getOption("floodam_building_stage")
	stage = intersect(as.character(unlist(stage)), getOption("floodam_building_stage"))

    if (length(stage) == 0) {
        if (verbose) message("No admissible stage asked. Nothing is done.\n")
        return(NULL)
    }

	# Perform hydraulic analysis
	if ("hydraulic" %in% stage) {
		# initial checks
		opening_scenario = match.arg(opening_scenario)
		if (missing(flood_duration)) {
			flood_duration = 2 * max(limnigraph[["limnigraph"]][, "time"]) + 1800
		}

		if (verbose) {
			message(
				sprintf("Simulating hydraulics for '%s'...\n", model[["name"]]),
				appendLF = FALSE
			)
		}
		
		# determine opening scenario
		data_hyd = prepare_hydraulic_input(
			model,
			external = limnigraph[["external"]]
		)
		opening = data_hyd[[opening_scenario]]

		# adding informations for the simulation
		opening[["discharge_coeff"]] = Cd
		opening[["H_top"]] = opening[["H_abs"]] + opening[["height"]]
		room = data_hyd[["room"]]

		# starting the simulation
		result = compute_hydraulic(
			opening = opening,
			room = room,
			bound = limnigraph[["limnigraph"]],
			dt_max = dt_max,
			t_start = 0,
			t_max = flood_duration
		)

		if (verbose) {
			message(
				sprintf(
					"\t... hydraulics successfully modeled for '%s'",
					model[["name"]]
				)
			)
		}
	}

	# Saving outputs
	if ("save" %in% stage) {
		name = sprintf(
			"%s-%s-%s-%s",
			model[['name']],
			flood_duration,
			opening_scenario,
			if (is.null(sim_id)) "NA" else sim_id
		)
		path = model[["path"]][["model_output_hydraulic"]]
		for (what in intersect(names(result), what)) {
			data.table::fwrite(
				as.data.frame(result[[what]]),
				file.path(path, sprintf("%s-%s.csv.gz", name, what))
			)
		}
			message(
				sprintf(
					"\t... hydraulics successfully saved for '%s' in '%s'",
					model[["name"]],
					path
				)
			)
	}

	duration = Sys.time() - start

	if (verbose) {
		message(
			sprintf(
				"End of analysis for '%s'. Total elapsed time %.2f %s\n",
				model[['name']],
				duration,
				attr(duration, "units")
			)
		)
	}

	invisible(result)
}
