#' Analyse the hydraulics of the maquette
#'
#' @param opening hydraulic part of the model, containing informations on the 
#' 		exchange
#' @param room hydraulic part of the model, containing informations on the 
#' 		rooms
#' @param bound numeric, matrix of floodwater depth heights by instant t
#' 		for each exterior opening exposed to flooding
#' @param dt_max numeric, optional; maxmimum timestep for simulation
#' @param t_start numeric, start time of the simulation
#' @param t_max numeric, duration of the flood event in seconds,
#' 		optional; if not provided, calculated using limnigraph
#'
#' @return lists of hydraulic values: water exchange through openings and
#' 	floodwater depth in rooms
#'
#' @export

compute_hydraulic = function(opening, room, bound, dt_max, t_start = 0, t_max) {

	exterior_name = colnames(bound)[-1]
	room_name = room[["room"]] = as.character(room[["room"]])
	opening_name = opening[["opening"]]
	building_name = c(room_name, exterior_name)

	# init building and boundaries
	building = rbind(
		surf = c(room[["surface"]],	rep(NA_real_, ncol(bound) - 1)),
		z0 = c(room[["H_abs"]], numeric(ncol(bound) - 1)),
		z = c(room[["initial_depth"]], bound[1, -1])
	)
	colnames(building) = building_name

	if (any(building["z", ] < 0)) stop("h cannot be negative")

	# init exchange
	exchange = rbind(
		width = opening[["width"]],
		z_bottom = opening[["H_abs"]],
		z_top = opening[["H_top"]], 
		Cd = opening[["discharge_coeff"]],
		z_up = 0,
		z_down = 0,
		flow = 0,
		s = 0
	)
	colnames(exchange) = opening_name

	# setting up the relations between rooms and exchange
	match_opening = t(as.matrix(opening[c("up", "down")]))
	colnames(match_opening) = opening_name
	matching = matrix(
		0, 
		nrow = nrow(opening),
		ncol = ncol(building),
		dimnames = list(opening_name, building_name)
	)
	for (o in opening_name) matching[o, match_opening[, o]] = c(-1, 1)

	# preallocating the output matrices
	nrows = floor(t_max / dt_max)
	result_h = matrix(
		NA_real_,
		nrow = nrows,
		ncol = ncol(building) + 1,
  		dimnames = list(c(), c("time", building_name)))
	result_q = matrix(
		NA_real_,
		nrow = nrows,
  		ncol = nrow(opening) + 1,
  		dimnames = list(c(), c("time", opening_name)))
	result_v = result_s = result_q
	result_h[1, 1] = result_q[1, 1] = result_s[1, 1] = result_v[1, 1] = t_start
	
	pos = 1

	# declaring the interpolation functions to compute water level on the outside
	approx_fun = sapply(
		exterior_name,
		function(x) {stats::approxfun(bound[, "time"], bound[, x], yright = 0)}
	)

	# time loop
	t_calc = t_start
	for (pos in seq(nrows)) {
		# get level for boundaries
		for (e in exterior_name) {
			z = approx_fun[[e]](t_calc)
			building["z", e] = z
		}
		# get level at exchange up and down
		exchange[c("z_up", "z_down"), ] = building["z", match_opening]
			
		# compute flow and section for all exchange
		exchange[c("s", "flow"), ] = rcpp_compute_exchange(exchange)
		# exchange[c("s", "flow"), ] = compute_exchange(exchange, opening_name)

		# compute dt_adj if needed
		dt_adj = dt_max
		Qtot = colSums(exchange["flow", ] * matching[, room_name])
		check = room_name[Qtot < 0]
		if (length(check) > 0) {
			volume_in = (building["z", check] - building["z0", check]) * building["surf", check]
			dt_adj = min(dt_adj, volume_in / -Qtot[check])
		}

		# get level for room
		building["z", room_name] = building["z", room_name] + Qtot * dt_adj / building["surf", room_name]
	
		# updating the output matrices
		result_h[pos, ] = c(t_calc, building["z", ])
		result_q[pos, ] = c(t_calc, exchange["flow", ])
		result_s[pos, ] = c(t_calc, exchange["s", ])
		result_v[pos, ] = c(t_calc, exchange["flow", ] / exchange["s", ])

		# time incrementation
		t_calc = t_calc + dt_adj
	}

	# returning output
	invisible(
		list(
			"h" = result_h,
			"eQ" = result_q,
			"eS" = result_s,
			"v" = result_v
		)
	)
}
