#' @title Build a limnigraph 
#' 
#' @description
#' `generate_limnigraph()` builds external limingraphs that are used as boundary
#' conditions for interior hydraulic simulation. It also gives the corresponding
#' openings to each limnigraph
#' 
#' @details
#' `time` gives the different time steps used for all limnigraphs. Its length is used as the
#' standard.
#' 
#' `depth` can be a vector of the same length as `time`. In that case only one limnigraph will
#' be produced. For more limnigraphs a matrix with colnames shall be used.
#' 
#' `external` gives corresponding openings to each limnigraphs (columns of `depth`). In case
#' only one limingraph is given, external can be directly a character vector of corresponding
#' opnenings.
#' 
#' @param time numeric, time steps (in seconds) at which a specific floodwater depth is reached
#' @param depth numeric, depths (in meters) reached at time steps given by `time`
#' @param external character or list of characters, giving the names of the openings concerned by external limnigraphs
#' @param verbose boolean, will floodam tells what it is doing, default to getOption("floodam_building_verbose")
#'
#' @return list, with a matrix containing limnigraphs, and a list of the matching openings
#' 
#' @examples
#' 
#' # Simpliest limnigraph (one external)
#' limnigraph = generate_limnigraph(
#' 	time = c(0, 1000, 2000),
#' 	depth = c(0, 1, 0),
#' 	external = "door"
#' )
#' 
#' # Limnigraph
#' limnigraph = generate_limnigraph(
#' 	time = c(0, 1000, 2000),
#' 	depth = cbind(ext_1 = c(0, 1, 0), ext_2 = 0),
#' 	external = list(ext_1 = "door", ext_2 = "window")
#' )
#' 
#' @export

generate_limnigraph = function(
		time,
		depth,
		external,
        verbose = getOption("floodam_building_verbose") 
	) {	
	if (verbose) message(sprintf("generating limnigraph ..."), appendLF = FALSE)
	if (is.null(ncol(depth))) {
		dim(depth) = c(length(depth), 1)
		colnames(depth) = "external"
	}
	if (!is.list(external) && ncol(depth) == 1) {
		external = list(external = external)
	}
	if (length(time) != nrow(depth)) {
		stop("'time' and 'depth' do not have compatible lengths.")
	}
	if (!identical(sort(colnames(depth)), sort(names(external)))) {
		stop("'depth' and 'external' names must be matching")
	}

    result = list(limnigraph = cbind(time, depth), external = c(external))

	if (verbose) message(sprintf(" limnigraph successfully generated"))

	return(result)
}
